## Overview

This charm provides computation and storage resources for an Apache Hadoop
deployment, and is intended to be used only as a part of that deployment.
This document describes how this charm connects to and interacts with the
other components of the deployment.


## Provided Relations

### resourcemanager (interface: mapred)

This relation connects this charm to the apache-hadoop-client charm.
The relation exchanges the following keys:

* Sent to the client:

  * `private-address`: Address of this unit, to provide the ResourceManager
  * `port`: Port upon which the ResourceManager is listening
  * `ready`: A flag indicating that YARN is ready to execute map-reduce jobs

* Received from the client:

  *There are no keys received from the client*

To use this interface, it is recommended that you use the relation class
provided in the [Big Data charmhelpers](http://bazaar.launchpad.net/~bigdata-dev/bigdata-data/trunk/files/head:/common/noarch/):

    from charmhelpers.contrib.bigdata.relations import NameNode
    from charmhelpers.core import unitdata

    resourcemanager = ResourceManager()
    if resourcemanager.is_ready():
        rm_units = unitdata.kv.get('relations.ready')['resourcemanager']
        rm_data = rm_units.values()[0]
        print rm_data['private-address']


## Required Relations

### namenode (interface: dfs)

This relation connects this charm to the apache-hadoop-hdfs-master charm.
The relation exchanges the following keys:

* Sent to hdfs-master:

  *There are no keys sent to hdfs-master*

* Received from hdfs-master:

  * `private-address`: Address of the HDFS master unit, to provide the NameNode
  * `port`: Port of the NameNode
  * `ready`: A flag indicating that HDFS is ready to store data


### nodemanager (interface: mapred-slave)

This relation connects this charm to the apache-hadoop-compute-slave charm.
The relation exchanges the following keys:

* Sent to compute-slave:

  * `private-address`: Address of the YARN master unit, to provide the ResourceManager
  * `port`: Port upon which the ResourceManager is listening
  * `ready`: A flag indicating that YARN is ready to register NodeManagers

* Received from compute-slave:

  * `private-address`: Address of the remote unit, to be registered as a NodeManager


## Manual Deployment

The easiest way to deploy the core Apache Hadoop platform is to use one of
the [apache-core-batch-processing-* bundles](https://jujucharms.com/q/bigdata-dev/apache?type=bundle).
However, to manually deploy the base Apache Hadoop platform without using one of the
bundles, you can use the following:

    juju deploy apache-hadoop-hdfs-master hdfs-master
    juju deploy apache-hadoop-hdfs-secondary secondary-namenode
    juju deploy apache-hadoop-yarn-master yarn-master
    juju deploy apache-hadoop-compute-slave compute-slave -n3
    juju deploy apache-hadoop-client client
    juju add-relation yarn-master hdfs-master
    juju add-relation secondary-namenode hdfs-master
    juju add-relation compute-slave yarn-master
    juju add-relation compute-slave hdfs-master
    juju add-relation client yarn-master
    juju add-relation client hdfs-master

This will create a scalable deployment with separate nodes for each master,
and a three unit compute slave (NodeManager and DataNode) cluster.  The master
charms also support co-locating using the `--to` option to `juju deploy` for
more dense deployments.
